
/*
 * apricot.c:
 *      A Programming Interactive Calculate O Tron...
 *
 *	Copyright (c) 2023 by Gordon Henderson
 *
 *********************************************************************************
 * This file is part of Apricot:
 *    A PRogramable Interactive Calculate O Tron by Gordon Henderson
 *
 *    Apricot is distributed under a "Source Available" license.
 *	It is NOT Open source and must not be treated as such.
 *
 *    See the file LICENSE for details.
 *********************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>


#include "readLineS.h"
#include "apricot.h"

#include "loadSave.h"


/*
 * eraseProgram:
 *	Delete. Delete. Delete. Delete
 *********************************************************************************
 */

void eraseProgram (int force)
{
  int i ;
  char line [10] ;

  if (!force)
  {
    if (numCodeLines == 0)
      { printf ("+++ No program to erase\n") ; return ; }

    if (!programEdited)
    {
      printf ("Erase program. Sure? ") ;
      readLineS (line, 10) ;
      if (! ((*line == 'Y') || (*line == 'y')))
      {
	printf ("Not erased.\n") ;
	return ;
      }
    }
  }

  for (i = 0 ; i < MAX_LINES ; ++i)
  {
    lineNumbers  [i] = 0 ;
    codePointers [i] = 0 ;
    if (codeLines [i] != NULL)
    {
      free (codeLines [i]) ;
      codeLines [i] = NULL ;
    }
  }

  if (!force)
    printf ("+++ Erased %d line%s\n", numCodeLines, numCodeLines == 1 ? "" : "s") ;

  numCodeLines = 0 ;
  allClear () ;
}


/*
 * loadProgram:
 *	Get a program from the filing system
 *********************************************************************************
 */

void loadProgram (void)
{
  char filename [80] ;
  char line     [80] ;
  FILE *fd ;
  int lineNum = 100 ;
  int len ;

  if (programEdited)
    { printf ("+++ Cowardly refusing to overwrite modified program.\n") ; return ; }

  if (numCodeLines != 0)
    eraseProgram (TRUE) ;

  printf ("Load program\n") ;
  printf ("  Filename: ") ;

  readLineS (filename, 78) ;
  if (strlen (filename) == 0)
    return ;

  if ((fd = fopen (filename, "r")) == NULL)
  {
    printf ("+++ Can't open %s: %s\n", filename, strerror (errno)) ;
    return ;
  }

  pEnd = 0 ;
  while (!feof (fd))
  {
    if (fgets (line, 78, fd) == NULL)
      break ;

// Dump trailing newlines, CRs or spaces

    len = strlen (line) ;
    while (len > 0)
    {
      char c = line [len - 1] ;

      if ( isspace (c) )
	line [--len] = 0 ;
      else
	break ;
    }

    insertLine (lineNum, line, 0) ;
    lineNum += 10 ;

    if (!compileLine (line))
    {
      fclose (fd) ;
      eraseProgram (TRUE) ;
      return ;
    }

  }

  fclose (fd) ;
}


/*
 * savedProgram:
 *	Store a program into the filing system.
 *********************************************************************************
 */

void saveProgram (void)
{
  char filename [80] ;
  FILE *fd ;
  int i ;

  if (numCodeLines == 0)
    { printf ("+++ No program to save.\n") ; return ; }

  printf ("Save program\n") ;
  printf ("  Filename: ") ;

  readLineS (filename, 78) ;
  if (strlen (filename) == 0)
    return ;

  if ((fd = fopen (filename, "w")) == NULL)
  {
    printf ("++ Can't open %s: %s\n", filename, strerror (errno)) ;
    return ;
  }

  for (i = 0 ; i < numCodeLines ; ++i)
    fprintf (fd, "%s\n", codeLines [i]) ;

  fclose (fd) ;

  printf ("+++ Wrote %d line%s\n", numCodeLines, numCodeLines == 1 ? "" : "s") ;
}
