
/*
 * readLineS.c:
 *      A Programming Interactive Calculate O Tron...
 *
 *	Copyright (c) 2023 by Gordon Henderson
 *
 *********************************************************************************
 * This file is part of Apricot:
 *    A PRogramable Interactive Calculate O Tron by Gordon Henderson
 *
 *    Apricot is distributed under a "Source Available" license.
 *	It is NOT Open source and must not be treated as such.
 *
 *    See the file LICENSE for details.
 *********************************************************************************
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

#include "version.h"

#include "tokens.h"
#include "loadSave.h"

#include "apricot.h"
#include "readLineS.h"


/*
 * readLineS:
 *	Read in a line of text, drop any trailing NL/CR and strip leading
 *	and/or trailing spaces.
 *	Also zero terminates the line.
 *********************************************************************************
 */

void readLineS (char *line, int maxLen)
{
  int i, len ;

  if (fgets (line, maxLen, stdin) == NULL)	// EOF?
    exit (1) ;

  if (strlen (line) == 1)			// Just a newline
  {
    *line = 0 ;
    return ;					// ... return empty string
  }

// Dump trailing newlines, CRs or spaces

  len = strlen (line) ;
  while (len > 1)
  {
    char c = line [len - 1] ;

    if ( isspace (c))
      line [--len] = 0 ;
    else
      break ;
  }

  if (len == 0)
    return ;

// Drop any leading spaces...

  while (isspace (line [0]))
  {
    for (i = 0 ; i < len ; ++i)
      line [i] = line [i+1] ;
    --len ;

    if (len == 0)
      { *line = 0 ; return ; }
  }
}
